/* linux/include/asm/arch-s3c2410/regs-lcd.h
 *
 * Copyright (c) 2003 Simtec Electronics <linux@simtec.co.uk>
 *		      http://www.simtec.co.uk/products/SWLINUX/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *
 *
 *  Changelog:
 *    12-06-2003     BJD     Created file
 *    26-06-2003     BJD     Updated LCDCON register definitions
 *    12-03-2004     BJD     Updated include protection
 *    10-03-2005     LCVR    Changed S3C2410_VA to S3C24XX_VA
*/


#ifndef ___ASM_ARCH_REGS_LCD_H
#define ___ASM_ARCH_REGS_LCD_H "$Id: lcd.h,v 1.3 2003/06/26 13:25:06 ben Exp $"

#define S3C2410_LCDREG(x) ((x) + S3C24XX_VA_LCD)

/* LCD control registers */
#define S3C2410_LCDCON1	    S3C2410_LCDREG(0x00)
#define S3C2410_LCDCON2	    S3C2410_LCDREG(0x04)
#define S3C2410_LCDCON3	    S3C2410_LCDREG(0x08)
#define S3C2410_LCDCON4	    S3C2410_LCDREG(0x0C)
#define S3C2410_LCDCON5	    S3C2410_LCDREG(0x10)

#define S3C2410_LCDCON1_CLKVAL(x)  ((x) << 8)
#define S3C2410_LCDCON1_MMODE	   (1<<7)
#define S3C2410_LCDCON1_DSCAN4	   (0<<5)
#define S3C2410_LCDCON1_STN4	   (1<<5)
#define S3C2410_LCDCON1_STN8	   (2<<5)
#define S3C2410_LCDCON1_TFT	   (3<<5)

#define S3C2410_LCDCON1_STN1BPP	   (0<<1)
#define S3C2410_LCDCON1_STN2GREY   (1<<1)
#define S3C2410_LCDCON1_STN4GREY   (2<<1)
#define S3C2410_LCDCON1_STN8BPP	   (3<<1)
#define S3C2410_LCDCON1_STN12BPP   (4<<1)

#define S3C2410_LCDCON1_TFT1BPP	   (8<<1)
#define S3C2410_LCDCON1_TFT2BPP	   (9<<1)
#define S3C2410_LCDCON1_TFT4BPP	   (10<<1)
#define S3C2410_LCDCON1_TFT8BPP	   (11<<1)
#define S3C2410_LCDCON1_TFT16BPP   (12<<1)
#define S3C2410_LCDCON1_TFT24BPP   (13<<1)

#define S3C2410_LCDCON1_ENVID	   (1)

#define S3C2410_LCDCON2_VBPD(x)	    ((x) << 24)
#define S3C2410_LCDCON2_LINEVAL(x)  ((x) << 14)
#define S3C2410_LCDCON2_VFPD(x)	    ((x) << 6)
#define S3C2410_LCDCON2_VSPW(x)	    ((x) << 0)

#define S3C2410_LCDCON3_HBPD(x)	    ((x) << 19)
#define S3C2410_LCDCON3_WDLY(x)	    ((x) << 19)
#define S3C2410_LCDCON3_HOZVAL(x)   ((x) << 8)
#define S3C2410_LCDCON3_HFPD(x)	    ((x) << 0)
#define S3C2410_LCDCON3_LINEBLANK(x)((x) << 0)

#define S3C2410_LCDCON4_MVAL(x)	    ((x) << 8)
#define S3C2410_LCDCON4_HSPW(x)	    ((x) << 0)
#define S3C2410_LCDCON4_WLH(x)	    ((x) << 0)

#define S3C2410_LCDCON5_BPP24BL	    (1<<12)
#define S3C2410_LCDCON5_FRM565	    (1<<11)
#define S3C2410_LCDCON5_INVVCLK	    (1<<10)
#define S3C2410_LCDCON5_INVVLINE    (1<<9)
#define S3C2410_LCDCON5_INVVFRAME   (1<<8)
#define S3C2410_LCDCON5_INVVD	    (1<<7)
#define S3C2410_LCDCON5_INVVDEN	    (1<<6)
#define S3C2410_LCDCON5_INVPWREN    (1<<5)
#define S3C2410_LCDCON5_INVLEND	    (1<<4)
#define S3C2410_LCDCON5_PWREN	    (1<<3)
#define S3C2410_LCDCON5_ENLEND	    (1<<2)
#define S3C2410_LCDCON5_BSWP	    (1<<1)
#define S3C2410_LCDCON5_HWSWP	    (1<<0)

/* framebuffer start addressed */
#define S3C2410_LCDSADDR1   S3C2410_LCDREG(0x14)
#define S3C2410_LCDSADDR2   S3C2410_LCDREG(0x18)
#define S3C2410_LCDSADDR3   S3C2410_LCDREG(0x1C)

#define S3C2410_LCDBANK(x)	((x) << 21)
#define S3C2410_LCDBASEU(x)	(x)

#define S3C2410_OFFSIZE(x)	((x) << 11)
#define S3C2410_PAGEWIDTH(x)	(x)

/* colour lookup and miscellaneous controls */

#define S3C2410_REDLUT	   S3C2410_LCDREG(0x20)
#define S3C2410_GREENLUT   S3C2410_LCDREG(0x24)
#define S3C2410_BLUELUT	   S3C2410_LCDREG(0x28)

#define S3C2410_DITHMODE   S3C2410_LCDREG(0x4C)
#define S3C2410_TPAL	   S3C2410_LCDREG(0x50)

/* interrupt info */
#define S3C2410_LCDINTPND  S3C2410_LCDREG(0x54)
#define S3C2410_LCDSRCPND  S3C2410_LCDREG(0x58)
#define S3C2410_LCDINTMSK  S3C2410_LCDREG(0x5C)
#define S3C2410_LPCSEL     S3C2410_LCDREG(0x60)

#define S3C2410_TFTPAL(x)  S3C2410_LCDREG((0x400 + (x)*4))

/* 2443 TFT LCD controller registers */
#define S3C2443_LCDREG(x) ((x) + S3C2443_VA_TFTLCD)

#define S3C2443_VIDCON0      S3C2443_LCDREG(0x00)
#define S3C2443_VIDCON1      S3C2443_LCDREG(0x04)
#define S3C2443_VIDTCON0     S3C2443_LCDREG(0x08)
#define S3C2443_VIDTCON1     S3C2443_LCDREG(0x0C)
#define S3C2443_VIDTCON2     S3C2443_LCDREG(0x10)
#define S3C2443_WINCON0      S3C2443_LCDREG(0x14)
#define S3C2443_WINCON1      S3C2443_LCDREG(0x18)
#define S3C2443_VIDOSD0A     S3C2443_LCDREG(0x28)
#define S3C2443_VIDOSD0B     S3C2443_LCDREG(0x2C)
#define S3C2443_VIDOSD0C     S3C2443_LCDREG(0x30)
#define S3C2443_VIDOSD1A     S3C2443_LCDREG(0x34)
#define S3C2443_VIDOSD1B     S3C2443_LCDREG(0x38)
#define S3C2443_VIDOSD1C     S3C2443_LCDREG(0x3C)
#define S3C2443_VIDW00ADD0B0 S3C2443_LCDREG(0x64)
#define S3C2443_VIDW00ADD0B1 S3C2443_LCDREG(0x68)
#define S3C2443_VIDW01ADD0   S3C2443_LCDREG(0x6C)
#define S3C2443_VIDW00ADD1B0 S3C2443_LCDREG(0x7C)
#define S3C2443_VIDW00ADD1B1 S3C2443_LCDREG(0x80)
#define S3C2443_VIDW01ADD1   S3C2443_LCDREG(0x84)
#define S3C2443_VIDW00ADD2B0 S3C2443_LCDREG(0x94)
#define S3C2443_VIDW00ADD2B1 S3C2443_LCDREG(0x98)
#define S3C2443_VIDW01ADD2   S3C2443_LCDREG(0x9C)
#define S3C2443_VIDINTCON    S3C2443_LCDREG(0xAC)
#define S3C2443_W1KEYCON0    S3C2443_LCDREG(0xB0)
#define S3C2443_W1KEYCON1    S3C2443_LCDREG(0xB4)
#define S3C2443_W2KEYCON0    S3C2443_LCDREG(0xB8)
#define S3C2443_W2KEYCON1    S3C2443_LCDREG(0xBC)
#define S3C2443_W3KEYCON0    S3C2443_LCDREG(0xC0)
#define S3C2443_W3KEYCON1    S3C2443_LCDREG(0xC4)
#define S3C2443_W4KEYCON0    S3C2443_LCDREG(0xC8)
#define S3C2443_W4KEYCON1    S3C2443_LCDREG(0xCC)
#define S3C2443_WIN0MAP      S3C2443_LCDREG(0xD0)
#define S3C2443_WIN1MAP      S3C2443_LCDREG(0xD4)
#define S3C2443_WPALCON      S3C2443_LCDREG(0xE4)
#define S3C2443_SYSIFCON0    S3C2443_LCDREG(0x130)
#define S3C2443_SYSIFCON1    S3C2443_LCDREG(0x134)
#define S3C2443_DITHMODE     S3C2443_LCDREG(0x138)
#define S3C2443_SYFCCON0     S3C2443_LCDREG(0x13C)
#define S3C2443_SYFCCON1     S3C2443_LCDREG(0x140)
#define S3C2443_SYFCCON2     S3C2443_LCDREG(0x140)
#define S3C2443_CPUTRIGCON2  S3C2443_LCDREG(0x160)
#define S3C2443_WIN0PAL      S3C2443_LCDREG(0x400)
#define S3C2443_WIN1PAL      S3C2443_LCDREG(0x800)
#endif /* ___ASM_ARCH_REGS_LCD_H */



